@@ -3,16 +3,16 @@
 # ==========================================================================================================
 #' @title S4 class for funGp model selection data structures
 #' @description This is the formal representation of the assembly of data structures delivered by the model
-#' selection routines in the \link[=funGp-package]{funGp package}. Gaussian process models are useful
-#' statistical tools in the modeling of complex input-output relationships. An Xfgpm object contains the
+#' selection routines in the \link[=funGp-package]{funGp package}. An \code{Xfgpm} object contains the
 #' trace of an optimization process, conducted to build Gaussian process models of outstanding performance.
 #' \itemize{
 #'  \item{\strong{Main methods}}{\cr
-#'        \link[funGp]{fgpm_factory}: structural optimization of funGp models \cr
+#'        \link[funGp]{fgpm_factory}: structural optimization of \code{fgpm} models,
+#'        creator of the \code{"Xfgpm"} class. \cr
 #'  }
 #'  \item{\strong{Plotters}}{\cr
-#'        \link[funGp]{plotX}: diagnostic plots for a fgpm_factory optimization and the selected model \cr
-#'        \link[funGp]{plotEvol}: plot of the evolution of the model selection algorithm in funGp \cr
+#'        \link[funGp]{plot,Xfgpm-method}: plot of the evolution of the algorithm with \code{which = "evolution"}
+#'        or of the absolute and relative quality of the optimized model with \code{which = "diag"}.
 #'  }
 #' }
 #'
@@ -36,7 +36,11 @@
 #'   evaluated by the algorithm.
 #' @slot details Object of class \code{"list"}. Further information about the parameters of the ant colony
 #'   optimization algorithm and the evolution of the fitness along the iterations.
-#'
+#' @slot sIn An object of class \code{"matrix"} containing a copy of
+#'     the provided scalar inputs.
+#' @slot fIn An object of class \code{"list"} containing a copy of
+#'     the provided functional inputs.
+#' @slot sOut An object of class \code{"matrix"} containing a copy of the provided outputs.
 #' @section Useful material:
 #' \itemize{
 #'  \item{\strong{Manual}}{
@@ -45,7 +49,7 @@
 #'  }}
 #' }
 #'
-#' @author José Betancourt, François Bachoc and Thierry Klein
+#' @author José Betancourt, François Bachoc, Thierry Klein and Jérémy Rohmer
 #'
 #' @include 1_fgpm_Class.R
 #' @include 3_ant_admin.R
@@ -64,7 +68,10 @@ setClass("Xfgpm",
            log.crashes = "antsLog",        # search method
            n.solspace = "numeric",         # search method
            n.explored = "numeric",         # search method
-           details = "list"                # search method
+           details = "list",               # search method
+           sIn = "matrix",
+           fIn = "list",
+           sOut = "matrix"
          ),
          validity = function(object) {TRUE})
 
@@ -75,14 +82,16 @@ setClass("Xfgpm",
 # ==========================================================================================================
 #' @rdname show-methods
 #' @aliases show,Xfgpm-method
+#' @method show Xfgpm
+#' @noRd
 setMethod("show", "Xfgpm", function(object) show.Xfgpm(object))
 
 show.Xfgpm <- function(object) {
-  cat("Structural optimization_________________\n\n")
+  cat("Structural optimization_________________\n")
 
   cat(paste("stat:", object@stat, "\n"))
   cat(paste("value:", format(object@fitness, digits = 3, nsmall = 4), "\n"))
-  cat(paste("n.explored:", object@n.explored, "\n\n"))
+  cat(paste("n.explored:", object@n.explored, "\n"))
 
   cat("For selected structure: object@structure\n")
   cat("For log of success: object@log.success\n")
@@ -100,38 +109,38 @@ show.Xfgpm <- function(object) {
 #'   configurations of a funGp model, and the consequent selection of a high quality configuration. funGp
 #'   currently relies on an ant colony based algorithm to perform this task. The algorithm defines the
 #'   solution space based on the levels of each structural parameter currently available in the
-#'   \link[funGp]{fgpm} function, and performs as smart exploration of it. More details on the algorithm are
+#'   \link[funGp]{fgpm} function, and performs a smart exploration of it. More details on the algorithm are
 #'   provided in a dedicated
 #'   \href{https://hal.archives-ouvertes.fr/hal-02532713}{technical report}.
 #'   funGp might evolve in the future to include improvements in the current algorithm or alternative
 #'   solution methods.
 #'
-#' @param sIn an optional matrix of scalar input values to train the model. Each column must match an input
+#' @param sIn An optional matrix of scalar input values to train the model. Each column must match an input
 #'   variable and each row a training point. Either scalar input coordinates (sIn), functional input
 #'   coordinates (fIn), or both must be provided.
-#' @param fIn an optional list of functional input values to train the model. Each element of the list must
-#'   be a matrix containing to the set of curves corresponding to one functional input. Either scalar input
+#' @param fIn An optional list of functional input values to train the model. Each element of the list must
+#'   be a matrix containing the set of curves corresponding to one functional input. Either scalar input
 #'   coordinates (sIn), functional input coordinates (fIn), or both must be provided.
-#' @param sOut a vector (or 1-column matrix) containing the values of the scalar output at the specified
+#' @param sOut A vector (or 1-column matrix) containing the values of the scalar output at the specified
 #'   input points.
-#' @param ind.vl an optional numerical matrix specifying which points in the three structures above should be
+#' @param ind.vl An optional numerical matrix specifying which points in the three structures above should be
 #'   used for training and which for validation. If provided, the optimization will be conducted in terms of
-#'   the hold-out Q2, which comes from training the model with a subset of the points, and then estimate the
+#'   the hold-out coefficient of determination Q², which comes from training the model with a subset of the points, and then estimating the
 #'   prediction error in the remaining points. In that case, each column of \emph{ind.vl} will be interpreted
 #'   as one validation set, and the multiple columns will imply replicates. In the simplest case,
 #'   \emph{ind.vl} will be a one-column matrix or simply an array, meaning that a simple replicate should be
 #'   used for each model configuration explored. If not provided, the optimization will be conducted in terms
-#'   of the leave-one-out cross-validation Q2, which for a total number of n observations, comes from training
+#'   of the leave-one-out cross-validation Q², which for a total number of n observations, comes from training
 #'   the model n times, each using n-1 points for training and the remaining one for validation. This procedure
-#'   is typically costly due to the large number of hyperparameters optimizations that should be conducted,
+#'   is typically costly due to the large number of hyperparameter optimizations that should be conducted,
 #'   nonetheless, fgpm_factory implements the virtual equations introduced by Dubrule (1983) for Gaussian
-#'   processes, which require a single hyperparameters optimization. See the reference below for more details.
-#' @param ctraints an optional list specifying the constraints of the structural optimization problem. Valid
+#'   processes, which require a single hyperparameter optimization. See the reference below for more details.
+#' @param ctraints An optional list specifying the constraints of the structural optimization problem. Valid
 #'   entries for this list are: \cr\cr
 #'   \strong{*}\emph{s_keepOn}: a numerical array indicating the scalar inputs that should remain active in the
-#'     model. It should contain the index of the columns of sIn corresponding to the inputs to keep active. \cr\cr
+#'     model. It should contain the indices of the columns of sIn corresponding to the inputs to keep active. \cr\cr
 #'   \strong{*}\emph{f_keepOn}: a numerical array indicating the functional inputs that should remain active in
-#'     the model. It should contain the index of the elements of fIn corresponding to the inputs to keep active. \cr\cr
+#'     the model. It should contain the indices of the elements of fIn corresponding to the inputs to keep active. \cr\cr
 #'   \strong{*}\emph{f_disTypes}: a list specifying the set of distances that should be tested for some
 #'     functional inputs. The values should be taken from the possibilities offered by the \link[funGp]{fgpm}
 #'     function for the argument \emph{f_disType} therein. Valid choices at this time are "L2_bygroup" and
@@ -159,12 +168,12 @@ show.Xfgpm <- function(object) {
 #'     values should be taken from the possibilities offered by the \link[funGp]{fgpm} function for the argument
 #'     \emph{kerType} therein. Valid choices at this time are "gauss", "matern5_2" and "matern3_2". If not
 #'     provided, all the available kernel functions will be tried.
-#' @param setup an optional list indicating the value for some parameters of the structural optimization
+#' @param setup An optional list indicating the value for some parameters of the structural optimization
 #'   algorithm. The ant colony optimization algorithm available at this time allows the following entries: \cr\cr
 #'   \strong{Initial pheromone load}\cr\cr
 #'     \strong{*}\emph{tao0}: a number indicating the initial pheromone load on links pointing out to the
 #'       selection of a distance type, a projection basis or a kernel type. Default is 0.1. \cr\cr
-#'     \strong{*}\emph{dop.s}: a number controlling how likely is to activate a scalar input. It operates on a
+#'     \strong{*}\emph{dop.s}: a number controlling how likely it is to activate a scalar input. It operates on a
 #'       relation of the type \eqn{A  = dop.s * I}, where \emph{A} is the initial pheromone load of links
 #'       pointing out to the activation of scalar inputs and \emph{I} is the initial pheromone load of links
 #'       pointing out to their inactivation. Default is 1. \cr\cr
@@ -173,9 +182,9 @@ show.Xfgpm <- function(object) {
 #'       function that determines the initial pheromone values on the links connecting the L2_byindex distance
 #'       with the projection dimension. Default are 2 and 1.4, respectively. \cr\cr
 #'   \strong{Local pheromone update}\cr\cr
-#'     \strong{*}\emph{rho.l}: a number specifying the pheromone evaporation rate. Default is 0.1 \cr\cr
+#'     \strong{*}\emph{rho.l}: a number specifying the pheromone evaporation rate. Default is 0.1. \cr\cr
 #'   \strong{Global pheromone update}\cr\cr
-#'     \strong{*}\emph{u.gbest}: a boolean indicating if at each iterations, the pheromone load on the links
+#'     \strong{*}\emph{u.gbest}: a boolean indicating if at each iteration, the pheromone load on the links
 #'       of the best ant of the whole trial should be reinforced. Default is FALSE. \cr\cr
 #'     \strong{*}\emph{n.ibest}: a number indicating how many top ants of each iteration should be used for
 #'       pheromone reinforcement. Default is 1. \cr\cr
@@ -189,29 +198,29 @@ show.Xfgpm <- function(object) {
 #'       the ant through the link with higher pheromone load; the second rule works based on probabilities which
 #'       are proportional to the pheromone load on the feasible links. The ants will randomly chose one of the two
 #'       rules at each time. They will opt for rule 1 with probability \emph{q0}. Default is 0.95.
-#' @param time.lim an optional number specifying a time limit in seconds to be used as stopping condition for the
+#' @param time.lim An optional number specifying a time limit in seconds to be used as stopping condition for the
 #'   structural optimization.
-#' @param nugget an optional variance value standing for the homogeneous nugget effect. A tiny nugget might help
+#' @param nugget An optional variance value standing for the homogeneous nugget effect. A tiny nugget might help
 #'   to overcome numerical problems related to the ill-conditioning of the covariance matrix. Default is 1e-8.
-#' @param n.starts an optional integer indicating the number of initial points to use for the optimization of the
+#' @param n.starts An optional integer indicating the number of initial points to use for the optimization of the
 #'   hyperparameters. A parallel processing cluster can be exploited in order to speed up the evaluation of
 #'   multiple initial points. More details in the description of the argument par.clust below. Default is 1.
-#' @param n.presample an optional integer indicating the number of points to be tested in order to select the
+#' @param n.presample An optional integer indicating the number of points to be tested in order to select the
 #'   n.starts initial points. The n.presample points will be randomly sampled from the hyper-rectangle defined by: \cr \cr
 #'   1e-10 \eqn{\le} \code{ls_s.hyp[i]} \eqn{\le} 2*max(\code{sMs[[i]]}), for i in 1 to the number of scalar inputs, \cr
 #'   1e-10 \eqn{\le} \code{ls_f.hyp[i]} \eqn{\le} 2*max(\code{fMs[[i]]}), for i in 1 to the number of functional inputs, \cr \cr
 #'   with  sMs and fMs the lists of distance matrices for the scalar and functional inputs, respectively. The value of
 #'   n.starts will be assigned to n.presample if this last is smaller. Default is 20.
-#' @param par.clust an optional parallel processing cluster created with the \code{\link[parallel]{makeCluster}}
+#' @param par.clust An optional parallel processing cluster created with the \code{\link[parallel]{makeCluster}}
 #'   function of the \link[=parallel]{parallel package}. If not provided, structural configurations are evaluated in
 #'   sequence.
-#' @param pbars an optional boolean indicating if progress bars should be displayed. Default is TRUE.
+#' @param pbars An optional boolean indicating if progress bars should be displayed.
 #'
 #' @return An object of class \linkS4class{Xfgpm} containing the data structures linked to the structural optimization
-#'   of a funGp model. It includes as the main component, an object of class \linkS4class{fgpm} corresponding to the
+#'   of a funGp model. It includes as the main component an object of class \linkS4class{fgpm} corresponding to the
 #'   optimized model. It is accessible through the \code{@@model} slot of the Xfgpm object.
 #'
-#' @author José Betancourt, François Bachoc and Thierry Klein
+#' @author José Betancourt, François Bachoc, Thierry Klein and Jérémy Rohmer
 #'
 #' @references Betancourt, J., Bachoc, F., Klein, T., Idier, D., Pedreros, R., and Rohmer, J. (2020),
 #' "Gaussian process metamodeling of functional-input code for coastal flood hazard assessment".
@@ -234,149 +243,114 @@ show.Xfgpm <- function(object) {
 #' \emph{Journal of the International Association for Mathematical Geology}, \strong{15},  687-699.
 #' \href{https://link.springer.com/article/10.1007/BF01033232}{[MG]}
 #'
-#' @seealso \strong{*} \link[funGp]{plotX} for diagnostic plots for a fgpm_factory output and selected model;
-#' @seealso \strong{*} \link[funGp]{plotEvol} for a plot of the evolution of the model selection algorithm in fgpm_factory;
-#' @seealso \strong{*} \link[funGp]{get_active_in} for post-processing of input data structures following a fgpm_factory call;
-#' @seealso \strong{*} \link[funGp]{predict} for predictions based on a funGp model;
-#' @seealso \strong{*} \link[funGp]{simulate} for simulations based on a funGp model;
-#' @seealso \strong{*} \link[funGp]{update} for post-creation updates on a funGp model.
+#' @seealso \strong{*} \link[funGp]{plot,Xfgpm-method} with
+#'     \code{which = "evolution"} for visualizing the evolution of
+#'     the ACO algorithm, or with \code{which = "diag"} for a
+#'     diagnostic plot;
+#' @seealso \strong{*} \link[funGp]{get_active_in} for post-processing
+#'     of input data structures following a fgpm_factory call;
+#' @seealso \strong{*} \link[funGp]{predict,fgpm-method} for
+#'     predictions based on a funGp model;
+#' @seealso \strong{*} \link[funGp]{simulate,fgpm-method} for
+#'     simulations based on a funGp model;
+#' @seealso \strong{*} \link[funGp]{update,fgpm-method} for
+#'     post-creation updates on a funGp model.
 #'
 #' @examples
-#' # calling fgpm_factory with the default arguments__________________________________________
-#' # generating input and output data
+#' #construction of a fgpm object
 #' set.seed(100)
 #' n.tr <- 32
-#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
-#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
-#'                    x5 = seq(0,1,length = n.tr^(1/5)))
-#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
+#' x1 <- x2 <- x3 <- x4 <- x5 <- seq(0,1,length = n.tr^(1/5))
+#' sIn <- expand.grid(x1 = x1, x2 = x2, x3 = x3, x4 = x4, x5 = x5)
+#' fIn <- list(f1 = matrix(runif(n.tr * 10), ncol = 10),
+#'             f2 = matrix(runif(n.tr * 22), ncol = 22))
 #' sOut <- fgp_BB7(sIn, fIn, n.tr)
-#' \dontrun{
-#' # optimizing the model structure with fgpm_factory (~12 seconds)
-#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut)
-#' plotLOO(xm@model) # plotting the model
 #'
-#' # building the model with the default fgpm arguments to compare
-#' m1 <- fgpm(sIn = sIn, fIn = fIn, sOut = sOut)
-#' plotLOO(m1) # plotting the model
+#' # optimizing the model structure with fgpm_factory (~12 seconds)
+#' \dontrun{
+#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut)
+#'}
 #'
 #' # assessing the quality of the model
 #' # in the absolute and also w.r.t. the other explored models
-#' plotX(xm)
+#' plot(xm, which = "diag")
 #'
 #' # checking the evolution of the algorithm
-#' plotEvol(xm)
-#' }
-#' \dontrun{
+#' plot(xm, which = "evol")
+#'
+#' # Summary of the tested configurations
+#' summary(xm)
+#'
+#' # checking the log of crashed iterations
+#' print(xm@log.crashes)
+#'
+#' # building the model with the default fgpm arguments to compare
+#' set.seed(100)
+#' n.tr <- 32
+#' x1 <- x2 <- x3 <- x4 <- x5 <- seq(0,1,length = n.tr^(1/5))
+#' sIn <- expand.grid(x1 = x1, x2 = x2, x3 = x3, x4 = x4, x5 = x5)
+#' fIn <- list(f1 = matrix(runif(n.tr * 10), ncol = 10),
+#' f2 <- matrix(runif(n.tr * 22), ncol = 22))
+#' sOut <- fgp_BB7(sIn, fIn, n.tr)
+#' m1 <- fgpm(sIn = sIn, fIn = fIn, sOut = sOut)
+#' plot(m1) # plotting the model
+#'
 #' # improving performance with more iterations_______________________________________________
-#' # generating input and output data
-#' set.seed(100)
-#' n.tr <- 32
-#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
-#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
-#'                    x5 = seq(0,1,length = n.tr^(1/5)))
-#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
-#' sOut <- fgp_BB7(sIn, fIn, n.tr)
-#'
-#' # default of 15 iterations (~12 seconds)
-#' xm15 <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut)
-#'
-#' # increasing to 25 iterations (~20 seconds)
-#' xm25 <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut, setup = list(n.iter = 25))
-#'
-#' # plotting both models
-#' plotLOO(xm15@model)
-#' plotLOO(xm25@model)
-#' }
+#' # call to fgpm_factory (~22 seconds)
 #' \dontrun{
+#' xm25 <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut,
+#'                      setup = list(n.iter = 25))
+#'}
+#'
+#' #assessing evolution and quality
+#' plot(xm25, which = "evol")
+#' plot(xm25, which = "diag")
+#'
 #' # custom solution space____________________________________________________________________
-#' # generating input and output data
-#' set.seed(100)
-#' n.tr <- 32
-#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
-#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
-#'                    x5 = seq(0,1,length = n.tr^(1/5)))
-#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
-#' sOut <- fgp_BB7(sIn, fIn, n.tr)
-#'
-#' # setting up the constraints
 #' myctr <- list(s_keepOn = c(1,2), # keep both scalar inputs always on
-#'               f_keepOn = c(2), # keep f2 always active
-#'               f_disTypes = list("2" = c("L2_byindex")), # only use L2_byindex distance for f2
-#'               f_fixDims = matrix(c(2,4), ncol = 1), # f2 projected in dimension 4
-#'               f_maxDims = matrix(c(1,5), ncol = 1), # f1 projected in dimension max 5
-#'               f_basTypes = list("1" = c("B-splines")), # only use B-splines projection for f1
-#'               kerTypes = c("matern5_2", "gauss")) # test only Matern 5/2 and Gaussian kernels
+#' f_keepOn = c(2), # keep f2 always active
+#' f_disTypes = list("2" = c("L2_byindex")), # only use L2_byindex distance for f2
+#' f_fixDims = matrix(c(2,4), ncol = 1), # f2 projected in dimension 4
+#' f_maxDims = matrix(c(1,5), ncol = 1), # f1 projected in dimension max 5
+#' f_basTypes = list("1" = c("B-splines")), # only use B-splines projection for f1
+#' kerTypes = c("matern5_2", "gauss")) # test only Matern 5/2 and Gaussian kernels
+#' #
+#' # call to fgpm_factory (~12 seconds)
+#' \dontrun{
+#' xmc <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut, ctraints = myctr)
+#'}
 #'
-#' # calling the funGp factory with specific constraints (~17 seconds)
-#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut, ctraints = myctr)
+#' #assessing evolution and quality
+#' plot(xmc, which = "evol")
+#' plot(xmc, which = "diag")
 #'
 #' # verifying constraints with the log of some successfully built models
-#' cbind(xm@log.success@sols, "Q2" = xm@log.success@fitness)
-#' }
-#' \dontrun{
+#' summary(xmc)
+#'
 #' # custom heuristic parameters______________________________________________________________
-#' # generating input and output data
-#' set.seed(100)
-#' n.tr <- 32
-#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
-#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
-#'                    x5 = seq(0,1,length = n.tr^(1/5)))
-#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
-#' sOut <- fgp_BB7(sIn, fIn, n.tr)
+#' mysup <- list(n.iter = 30, n.pop = 12, tao0 = .15, dop.s = 1.2,
+#'               dop.f = 1.3, delta.f = 4, dispr.f = 1.1, q0 = .85,
+#'               rho.l = .2, u.gbest = TRUE, n.ibest = 2, rho.g = .08)
 #'
-#' # defining the heuristic parameters
-#' mysup <- list(n.iter = 30, n.pop = 12, tao0 = .15, dop.s = 1.2, dop.f = 1.3, delta.f = 4,
-#'               dispr.f = 1.1, q0 = .85, rho.l = .2, u.gbest = TRUE, n.ibest = 2, rho.g = .08)
-#'
-#' # calling the funGp factory with a custom heuristic setup (~17 seconds)
-#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut, setup = mysup)
+#' # call to fgpm_factory (~20 seconds)
+#' \dontrun{
+#' xmh <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut, setup = mysup)
+#' }
 #'
 #' # verifying heuristic setup through the details of the Xfgpm object
-#' unlist(xm@details$param)
-#' }
-#' \dontrun{
+#' unlist(xmh@details$param)
+#'
 #' # stopping condition based on time_________________________________________________________
-#' # generating input and output data
-#' set.seed(100)
-#' n.tr <- 32
-#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
-#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
-#'                    x5 = seq(0,1,length = n.tr^(1/5)))
-#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
-#' sOut <- fgp_BB7(sIn, fIn, n.tr)
-#'
-#' # setting up a sufficiently large number of iterations
 #' mysup <- list(n.iter = 2000)
-#'
-#' # defining time budget
 #' mytlim <- 60
 #'
-#' # calling the funGp factory with time limit (~60 seconds)
-#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut, setup = mysup, time.lim = mytlim)
-#' }
+#' # call to fgpm_factory (~60 seconds)
 #' \dontrun{
-#' # passing fgpm arguments through fgpm_factory______________________________________________
-#' # generating input and output data
-#' set.seed(100)
-#' n.tr <- 32
-#' sIn <- expand.grid(x1 = seq(0,1,length = n.tr^(1/5)), x2 = seq(0,1,length = n.tr^(1/5)),
-#'                    x3 = seq(0,1,length = n.tr^(1/5)), x4 = seq(0,1,length = n.tr^(1/5)),
-#'                    x5 = seq(0,1,length = n.tr^(1/5)))
-#' fIn <- list(f1 = matrix(runif(n.tr*10), ncol = 10), f2 = matrix(runif(n.tr*22), ncol = 22))
-#' sOut <- fgp_BB7(sIn, fIn, n.tr)
-#'
-#' # calling the funGp factory with custom fgpm parameters (~25 seconds)
-#' xm <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut,
-#'                    nugget = 0, n.starts = 3, n.presample = 12)
-#'
-#' # NOTE: in the run above, some models crash. This happens because we set the nugget to 0
-#' #       and some input points become duplicates when some variables are removed from
-#' #       the model. We strongly recommend to always run fgpm_factory with at least a
-#' #       small nugget in order to prevent loss of configurations. By default fgpm_factory
-#' #       runs with 1e-8, which is enough in most cases.
-#' xm@log.crashes
+#' xms <- fgpm_factory(sIn = sIn, fIn = fIn, sOut = sOut,
+#'                     setup = mysup, time.lim = mytlim)
 #' }
+#' summary(xms)
+#'
 #' \dontrun{
 #' # parallelization in the model factory_____________________________________________________
 #' # generating input and output data
@@ -396,7 +370,7 @@ show.Xfgpm <- function(object) {
 #' # NOTE: in order to provide progress bars for the monitoring of time consuming processes
 #' #       ran in parallel, funGp relies on the doFuture and future packages. Parallel processes
 #' #       suddenly interrupted by the user tend to leave corrupt connections. This problem is
-#' #       originated outside funGp, which limits our control over it. On section 4.1 of the
+#' #       originated outside funGp, which limits our control over it. In the manual
 #' #       of funGp, we provide a temporary solution to the issue and we remain attentive in
 #' #       case it appears a more elegant way to handle it or a manner to suppress it.
 #' #
@@ -409,7 +383,7 @@ show.Xfgpm <- function(object) {
 fgpm_factory <- function(sIn = NULL, fIn = NULL, sOut = NULL, ind.vl = NULL,
                           ctraints = list(), setup = list(), time.lim = Inf,
                           nugget = 1e-8, n.starts = 1, n.presample = 20,
-                          par.clust = NULL, pbars = TRUE) {
+                          par.clust = NULL, pbars = interactive()) {
 
   # launch timer
   time.str <- Sys.time()
@@ -438,10 +412,12 @@ fgpm_factory <- function(sIn = NULL, fIn = NULL, sOut = NULL, ind.vl = NULL,
   X.model@log.success <- opt$log.suc
   X.model@log.crashes <- opt$log.cra
   X.model@n.solspace <- getSpacesize(solspace$sp.user)
-  X.model@n.explored <- nrow(opt$log.suc@sols)
-  X.model@details <- opt$all.details
-
-  return(X.model)
+    X.model@n.explored <- nrow(opt$log.suc@sols)
+    X.model@details <- opt$all.details
+    X.model@sIn <- as.matrix(sIn)
+    X.model@fIn <- fIn
+    X.model@sOut <- as.matrix(sOut)
+    return(X.model)
 }
 # ==========================================================================================================
 
@@ -746,4 +722,243 @@ printSpace <- function(ds, df, space) {
   }
   cat("\n")
 }
-# ==========================================================================================================
+
+## =============================================================================
+## summary method
+## =============================================================================
+
+##' @description Display a summary of the structure of a \code{Xfgpm}
+##'     object, with a short description of up to \code{n} \code{fgpm}
+##'     objects visited during the ACO optimization.
+##'
+##' @details The displayed information depends on the number of
+##'     candidate inputs, in order to maintain compact tables.  The
+##'     inputs are labelled with integer suffixes, the prefix being
+##'     \code{"X"} for scalar inputs and \code{"F"} for functional
+##'     inputs.
+##'     \itemize{
+##'         \item{With a small number of inputs, the list
+##'               contains only one data frame. For each candidate
+##'               input (either scalar or functional) a column with
+##'               the input name indicates if the input
+##'               is active (cross \code{x}) or not (white space)
+##'               in the \code{fgpm} object corresponding to the row. For each
+##'               functional variable also shown are: the distance used \code{D_},
+##'               the dimension \code{Bas_} after dimension reduction,
+##'               the type of basis used \code{B_}. Remind that the
+##'               kernel (\code{Kern}) is the same for all functional
+##'               inputs. Also shown is the value of the Leave-One-Out
+##'               coefficient \code{Q²}.
+##'         }
+##'         \item{With a large number of inputs, the list contains
+##'               two data frames. The first one tells which inputs
+##'               are active among the scalar and functional candidate
+##'               inputs. The second data frame gives more details
+##'               for functional inputs as before.}
+##'       }
+##'
+##' @title Summary method for \code{Xfgpm} objects
+##' @param object A \code{Xfgpm} object.
+##' @param n Maximal number of lines (\code{fgpm} objects) to show.
+##' @param ... Not used yet.
+##'
+##' @return An object inheriting from \code{list}, actually a list
+##'     containing one or two data frames depending on the number of
+##'     inputs. In each data frame, the \code{n} rows provide
+##'     information on the best \code{fgpm} objects visited.
+##'
+##' @method summary Xfgpm
+##' @export
+##' @examples
+##' summary(xm)
+setMethod("summary", "Xfgpm",
+          function(object, n = 24, ...) {
+              summary.Xfgpm(object = object, n = n, ...)
+          })
+
+summary.Xfgpm <- function(object, n = 24, ...) {
+
+    ds <- ncol(object@sIn)
+    df <- length(object@fIn)
+    n <- pmin(n, nrow(object@log.success@sols))
+
+    ## =========================================================================
+    ## When the number of inputs is small enough, the structural
+    ## parameters can be displayed in a single data frame. With more variables
+    ## we split the content in two data frames (daf): one daf for
+    ## =========================================================================
+
+    if (4 * ds + 22 * df < 80) {
+        daf <- formatShort(object@log.success@sols)
+        daf <- cbind(daf,
+                     "Q2" = sprintf("%5.3f", object@log.success@fitness))
+        n <- pmin(n, nrow(daf))
+        daf <- daf[1L:n, , drop = FALSE]
+        daf <- list("Inputs and details" = daf)
+    } else {
+        fullDf <- object@log.success@sols
+        nms <- colnames(fullDf)
+        ## extract columns corresponding to the states
+        indStateScal <-  grep("State_X[0-9]*", names(fullDf))
+        indStateFun <-  grep("State_F[0-9]*", names(fullDf))
+        indKernel <-  grep("Kernel", names(fullDf))
+        activeDf <- formatShort(fullDf[ , c(indStateScal, indStateFun, indKernel),
+                                       drop = FALSE])
+        activeDf <- cbind(activeDf,
+                          "Q2" = sprintf("%5.3f", object@log.success@fitness))
+        activeDf <- activeDf[1L:n, , drop = FALSE]
+
+        indDf <-  grep("_F[0-9]*", names(fullDf))
+        funDf <- formatShort(fullDf[ , c(indDf, indKernel), drop = FALSE])
+        funDf <- cbind(funDf,
+                       "Q2" = sprintf("%5.3f", object@log.success@fitness))
+        funDf <- funDf[1L:n, , drop = FALSE]
+        daf <- list("State of inputs" = activeDf,
+                    "Details for functional inputs " = funDf)
+    }
+
+    class(daf) <- c("summary.Xfgpm", "list")
+    daf
+}
+
+## =============================================================================
+## print method
+## =============================================================================
+##' @method print summary.Xfgpm
+##' @export
+print.summary.Xfgpm <- function(x, ...) {
+    cat(names(x)[1], "\n")
+    print(x[[1]])
+    if (length(x) >= 2) {
+        cat(names(x)[2], "\n")
+        nBlocks <- ceiling(ncol(x[[2]]) / 12)
+        for (i in 1:nBlocks) {
+            iCol <- ((i - 1) * 12 + 1):min((i * 12), ncol(x[[2]]))
+            print(x[[2]][ , iCol])
+        }
+    }
+}
+
+## =============================================================================
+## Re fit a fgpm model
+## =============================================================================
+##' Refit a \code{fgpm} model as described in a \code{Xfgpm} object.
+##'
+##' @title Refit a \code{fgpm} model in a \code{Xfgpm} object
+##'
+##' @param x A \code{Xfgpm} object.
+##'
+##' @param i An integer giving the index of the model to refit. The
+##'     models are in decreasing fit quality as assessed by the
+##'     Leave-One-Out \eqn{Q^2}{Q2}.
+##'
+##' @section Caution: While the syntax may suggest that the function
+##'     \emph{extracts} a fitted \code{fgpm} model, this not true. The
+##'     \code{fgpm} model is refitted using the call that was used
+##'     when this model was assessed. The refitted \code{fgpm} model
+##'     keeps the same structural parameters as the one assessed
+##'     (active variables, kernel, ...), but since the optimization
+##'     uses random initial values, the optimized hyper-parameters may
+##'     differ from those of the corresponding \code{fgpm} in the
+##'     \code{Xfgpm} object \code{x}. As a result, the model can be
+##'     different and show a different LOO performance.
+##'
+##' @note The slot \code{@model} returns the best \code{fgpm} as
+##'     assessed in a \code{Xfgm} model \code{x}. So this model can be
+##'     expected to be close to the same as \code{x[[1]]}. Yet due to
+##'     the refit, the two models \code{x@model} and \code{x[[1]]} can
+##'     differ, see the explanations in the \bold{Caution} section.
+##'
+##' @export
+##' @method [[ Xfgpm
+##'
+##' @seealso The \code{\link{modelDef}} function to extract the
+##'     definition of a \code{fgpm} model e.g., to evaluate it using new
+##'     data \code{sIn}, \code{fIn} and \code{sOut}.
+##'
+##' @examples
+##' ## see `?xm` to see how to recreate the pre-caclulated `Xfgpm` object `xm`.
+##' xm[[2]]
+##'
+setMethod("[[", "Xfgpm",
+          function(x, i) {
+              if (i > length(x@log.success@args)) {
+                  stop("'i' must be less or equal the number of 'fgpm' ",
+                       "models fitted in 'x'")
+              }
+              ## copy the the inputs 'sIn' and 'fIn' stored in the
+              ## object 'x' into the current environnement. This is
+              ## necessary because these objects may have changed in
+              ## the global environment.
+              sIn <- x@sIn
+              fIn <- x@fIn
+              sOut <- x@sOut
+              text <- x@log.success@args[[i]]@string
+              eval(parse(text = text)[[1]])
+          })
+
+## =============================================================================
+## Retrieve a fgpm model from a Xfgpm object
+## =============================================================================
+
+##' Retrieve the \code{fgpm} model with index (or rank) \code{i} from
+##' within a \code{Xfgpm} object. By evaluating this code in an
+##' environment containing suitable objects \code{sIn}, \code{fIn} and
+##' \code{sOut} we can re-create a \code{fgpm} object.
+##'
+##' The models are sorted by decreasing quality so \code{i = 1} extracts
+##' the definition of the best model.
+##'
+##' @title Retrieve a \code{fgpm} from within a \code{Xfgpm} object
+##'
+##' @param object A \code{Xfgpm} object as created by
+##' \code{\link{fgpm_factory}}.
+##'
+##' @param ind The index (or rank) of the model in \code{object}.
+##'
+##' @return A parsed R code defining the \code{fgpm} model.
+##'
+##' @note Remind that the models are sorted by decreasing quality so
+##'     \code{i = 1} extracts the definition of the best model.
+##'
+##' @seealso The \code{\link{[[,Xfgpm-method}} that can also be used
+##'     to re-create a \code{fgpm} object using \emph{the same data}
+##'     as that used to create the \code{Xfgpm} object in
+##'     \code{object}.
+##'
+##' @export
+##'
+##' @examples
+##' ## =========================================================================
+##' ## Using the pre-calculated object `xm` to save time. See `?xm` to re-create
+##' ## this object.
+##' ## =========================================================================
+##'
+##' ## 'xm@model' is the best 'fgpm' model in 'xm'
+##' plot(xm@model)
+##'
+##' ## see the R code to use to recreate the model
+##' modelDef(xm, i = 1)
+##'
+##'\dontrun{
+##'     ## Define new data in a list. Using an environment would also work,
+##'     ## including the global environment, which is the default in  `eval`.
+##'     L <- list()
+##'     set.seed(341)
+##'     n.new <- 3^5
+##'     x1 <- x2 <- x3 <- x4 <- x5 <- seq(0, 1, length = n.new^(1/5))
+##'
+##'     ## create the data objects required to fit the model
+##'     L$sIn <- as.matrix(expand.grid(x1 = x1, x2 = x2, x3 = x3, x4 = x4, x5 = x5))
+##'     L$fIn <- list(f1 = matrix(runif(n.new * 10), ncol = 10),
+##'                    f2 = matrix(runif(n.new * 22), ncol = 22))
+##'     L$sOut <- fgp_BB7(L$sIn, L$fIn, n.new)
+##'
+##'     ## Now evaluate
+##'     fgpm.new <- eval(modelDef(xm, i = 1), envir = L)
+##'     plot(fgpm.new, main = "Re-created 'fgpm' model with different data")
+##'     plot(xm[[1]], main = "Re-created 'fgpm' model with the same data")
+##' }
+modelDef <- function(object, ind) {
+    parse(text = object@log.success@args[[ind]]@string[[1]])
+}
